use std::collections::HashMap;

use crate::utils::input_reader;
use crate::year_2023::day_08_haunted_wasteland::common::{parse_line, Node};

pub fn part_two() -> String {
    if let Ok(lines) = input_reader::read_lines("./input/2023/day_08/input.txt") {
        let mut node_map: HashMap<String, Node> = HashMap::new();
        let mut start_nodes: Vec<String> = Vec::new();
        let mut instructions: Vec<char> = Vec::new();
        let mut s: String;

        for (i, line) in lines.enumerate() {
            s = line.unwrap();

            if i == 0 {
                instructions = s.chars().collect();
                continue;
            }

            if s.is_empty() {
                continue;
            }

            let node = parse_line(&s);
            let value = node.value.clone();

            node_map.insert(value.clone(), node);

            if value.ends_with('A') {
                start_nodes.push(value.clone())
            }
        }

        let mut counts: Vec<usize> = Vec::new();

        for curr in start_nodes {

            let mut curr_node = &node_map[&curr];
            let mut count = 0;

            while !curr_node.value.ends_with('Z') {
                let i = count % instructions.len();

                if instructions[i] == 'L' {
                    curr_node = &node_map[&curr_node.left];
                } else if instructions[i] == 'R' {
                    curr_node = &node_map[&curr_node.right];
                }

                count += 1;
            }

            counts.push(count);
        }

        let result = counts.iter().fold(1, |acc, &x| lcm(acc, x));

        return result.to_string();
    } else {
        panic!("Failed to read lines from input file");
    }
}

fn lcm(a : usize, b : usize) -> usize {
    a * b / gcd(a, b)
}

fn gcd(a : usize, b : usize) -> usize {

    if b > a {
        return gcd(b, a);
    }
    
    if b == 0 {
        a
    } else {
        gcd(b, a % b)
    }
}