use super::common::Pipe;
use super::common::Point;
use super::common::Direction;

pub fn part_one(input_lines: Vec<String>) -> String {
    let vert_dim = input_lines.len();
    let horz_dim = input_lines[0].len();

    let mut matrix = vec![vec![Pipe::None; horz_dim]; vert_dim];
    let mut start_pos: Point = Point { x: 0, y: 0 };

    for (y, line) in input_lines.iter().enumerate() {
        for (x, c) in line.chars().enumerate() {
            match c {
                '|' => {
                    matrix[x][y] = Pipe::NorthSouth;
                }
                'J' => {
                    matrix[x][y] = Pipe::NorthWest;
                }
                'L' => {
                    matrix[x][y] = Pipe::NorthEast;
                }
                '-' => {
                    matrix[x][y] = Pipe::EastWest;
                }
                '7' => {
                    matrix[x][y] = Pipe::SouthWest;
                }
                'F' => {
                    matrix[x][y] = Pipe::SouthEast;
                }
                '.' => {
                    matrix[x][y] = Pipe::None;
                }
                'S' => {
                    matrix[x][y] = Pipe::Start;
                    start_pos = Point { x, y };
                }
                _ => panic!("Invalid character in input"),
            }
        }
    }

    let mut next: Option<Point> = None; 

    if start_pos.x > 0 {
        let pos = matrix[start_pos.x - 1][start_pos.y];
        match pos {
            Pipe::NorthEast => next = Some(Point { x: start_pos.x - 1, y: start_pos.y }),
            Pipe::SouthEast => next = Some(Point { x: start_pos.x - 1, y: start_pos.y }),
            Pipe::EastWest => next = Some(Point { x: start_pos.x - 1, y: start_pos.y }),
            _ => {}            
        }
    }

    if next.is_none() && start_pos.x < horz_dim - 1 {
        let pos = matrix[start_pos.x + 1][start_pos.y];
        match pos {
            Pipe::NorthWest => next = Some(Point { x: start_pos.x + 1, y: start_pos.y }),
            Pipe::SouthWest => next = Some(Point { x: start_pos.x + 1, y: start_pos.y }),
            Pipe::EastWest => next = Some(Point { x: start_pos.x + 1, y: start_pos.y }),
            _ => {}            
        }
    }

    if next.is_none() && start_pos.y > 0 {
        let pos = matrix[start_pos.x][start_pos.y - 1];
        match pos {
            Pipe::NorthWest => next = Some(Point { x: start_pos.x, y: start_pos.y - 1 }),
            Pipe::NorthEast => next = Some(Point { x: start_pos.x, y: start_pos.y - 1 }),
            Pipe::NorthSouth => next = Some(Point { x: start_pos.x, y: start_pos.y - 1 }),
            _ => {}            
        }
    }

    if next.is_none() && start_pos.y < vert_dim - 1 {
        let pos = matrix[start_pos.x][start_pos.y + 1];
        match pos {
            Pipe::SouthWest => next = Some(Point { x: start_pos.x, y: start_pos.y + 1 }),
            Pipe::SouthEast => next = Some(Point { x: start_pos.x, y: start_pos.y + 1 }),
            Pipe::NorthSouth => next = Some(Point { x: start_pos.x, y: start_pos.y + 1 }),
            _ => {}            
        }
    }

    let diff_x = start_pos.x as i32 - next.unwrap().x as i32;
    let diff_y = start_pos.y as i32 - next.unwrap().y as i32;

    // println!("Start: ({}, {})", start_pos.x, start_pos.y);
    // println!("Next: ({}, {})", next.unwrap().x, next.unwrap().y);
    // println!("Diff: ({}, {})", diff_x, diff_y);

    let mut direction_from = match (diff_x, diff_y) {    
        (1, 0) => Direction::East,
        (-1, 0) => Direction::West,
        (0, 1) => Direction::South,
        (0, -1) => Direction::North,    
        _ => panic!("Invalid direction"),
    };

    let mut count = 1;

    while next.unwrap() != start_pos {
        let x = next.unwrap().x;
        let y = next.unwrap().y;
        let pos = matrix[x][y];

        // println!("X, Y: ({}, {})", x, y);
        // println!("Direction: {:?}", direction_from);
        // println!("Pos: {:?}", pos);
        // println!();

        let mut next_direction_from = direction_from;

        match (direction_from, pos) {
            (Direction::North, Pipe::NorthSouth) => {
                next = Some(Point { x: x, y: y + 1 });
            }
            (Direction::North, Pipe::NorthEast) => {
                next = Some(Point { x: x + 1, y: y });
                next_direction_from = Direction::West;
            }
            (Direction::North, Pipe::NorthWest) => {
                next = Some(Point { x: x - 1, y: y });
                next_direction_from = Direction::East;
            }
            (Direction::East, Pipe::EastWest) => {
                next = Some(Point { x: x - 1, y: y });
            }
            (Direction::East, Pipe::NorthEast) => {
                next = Some(Point { x: x, y: y - 1 });
                next_direction_from = Direction::South;
            }
            (Direction::East, Pipe::SouthEast) => {
                next = Some(Point { x: x, y: y + 1 });
                next_direction_from = Direction::North;
            }
            (Direction::South, Pipe::SouthWest) => {
                next = Some(Point { x: x - 1, y: y });
                next_direction_from = Direction::East;
            }
            (Direction::South, Pipe::NorthSouth) => {
                next = Some(Point { x: x, y: y - 1 });
            }
            (Direction::South, Pipe::SouthEast) => {
                next = Some(Point { x: x + 1, y: y });
                next_direction_from = Direction::West;
            }
            (Direction::West, Pipe::EastWest) => {
                next = Some(Point { x: x + 1, y: y });
            }
            (Direction::West, Pipe::SouthWest) => {
                next = Some(Point { x: x, y: y + 1 });
                next_direction_from = Direction::North;
            }
            (Direction::West, Pipe::NorthWest) => {
                next = Some(Point { x: x, y: y - 1 });
                next_direction_from = Direction::South;
            }
            _ => panic!("Invalid direction"),            
        };

        direction_from = next_direction_from;        
        count += 1;
    }

    let result = count / 2 + count % 2;
    result.to_string()
}