use std::collections::HashMap;

use super::common::Spring;

pub fn part_two(input_lines: Vec<String>) -> String {

    let mut springs: Vec<Vec<Spring>> = Vec::new();
    let mut values: Vec<Vec<usize>> = Vec::new();

    for line in input_lines {

        let split_line: Vec<&str> = line.split(' ').collect();
        
        springs.push(parse_springs(split_line[0].to_string()));
        values.push(parse_nums(split_line[1].to_string()));
    }

    let mut res = 0;
    let mut memory: HashMap<(Vec<Spring>, Vec<usize>), usize> = HashMap::new();

    for i in 0..springs.len() {
        let search_spring = springs[i].clone();
        let val = search(search_spring, values[i].clone(), &mut memory);        
        res += val;
    }

    res.to_string()
}

fn parse_springs(line: String) -> Vec<Spring> {
    let mut spring_line: Vec<Spring> = Vec::new();

    for c in line.chars() {
        match c {
            '.' => spring_line.push(Spring::O),
            '#' => spring_line.push(Spring::D),
            _ => spring_line.push(Spring::U),
        }
    }

    let mut spring_res = <Vec<Spring>>::new();

    for i in 0..5 {
        spring_res.extend_from_slice(&spring_line);

        if i != 4 {
            spring_res.push(Spring::U);
        }
    }

    spring_res
    // spring_line
}

fn parse_nums(line: String) -> Vec<usize> {
    let nums: Vec<usize> = line.split(',').map(|x| x.parse().unwrap()).collect();
    let mut res_nums: Vec<usize> = Vec::new();

    for _ in 0..5 {
        res_nums.extend_from_slice(&nums);
    }

    res_nums
    // nums
}


fn search(spring_line: Vec<Spring>, groups: Vec<usize>, memory: &mut HashMap<(Vec<Spring>, Vec<usize>), usize>) -> usize {

    if memory.contains_key(&(spring_line.clone(), groups.clone())) {
        return memory[&(spring_line.clone(), groups.clone())];
    }

    let mut res = 0;

    if groups.len() == 0 {
        let damaged: Vec<usize> = spring_line.iter().enumerate().filter(|(_, x)| **x == Spring::D).map(|(i, _)| i).collect();
        if damaged.is_empty(){
            return 1;
        }
    } else if !spring_line.is_empty() && spring_line.len() >= groups[0] {

        match spring_line[0] {
            Spring::O => {
                res += search(spring_line[1..].to_vec(), groups.clone(), memory);
            },
            Spring::D => {
                let group = groups[0];

                let mut spring_group = spring_line[0..group].to_vec();
                
                for i in 0..group {
                    if spring_group[i] == Spring::U {
                        spring_group[i] = Spring::D;
                    }
                }
    
                let operational: Vec<usize> = spring_group.iter().enumerate().filter(|(_, x)| **x == Spring::O).map(|(i, _)| i).collect();
                if operational.len() == 0 {
                    if spring_line.len() == group {
                        if groups.len() == 1 {
                            return 1;
                        }
                    } else {
                        if spring_line[group] == Spring::O || spring_line[group] == Spring::U {
                            res += search(spring_line[group+1..].to_vec(), groups[1..].to_vec(), memory);                    
                        }
                    }
                }
            }
            _ => {
                let mut val;

                let mut op_clone = spring_line.clone();
                op_clone[0] = Spring::O;
                let op_groups = groups.clone();
                val = search(op_clone, op_groups, memory);
                res += val;
    
                let mut dmg_clone = spring_line.clone();
                dmg_clone[0] = Spring::D;
                let dmg_groups = groups.clone();
                val = search(dmg_clone, dmg_groups, memory);
                res += val;
            }
        }
    }

    memory.insert((spring_line.clone(), groups.clone()), res);
    res
}