pub fn part_one(input_lines: Vec<String>) -> String {

    let mut patterns: Vec<Vec<Vec<bool>>> = Vec::new();
    let mut acc: Vec<Vec<bool>> = Vec::new();

    for line in input_lines {
        if line.is_empty() && !acc.is_empty() {
            patterns.push(acc.clone());
            acc = Vec::new();
            continue;
        }

        let mut line_vec: Vec<bool> = Vec::new();
        for c in line.chars() {
            if c == '#' {
                line_vec.push(true);
            } else {
                line_vec.push(false);
            }
        }
        acc.push(line_vec);
    }

    if !acc.is_empty() {
        patterns.push(acc);
    }

    let mut result = 0;

    // println!("Found {} patterns", patterns.len());

    for pattern in patterns {
        let val = process_pattern(&pattern);
        result += val;
    }

    result.to_string()
    // String::from("Not implemented")
}

fn find_same_rows(pattern: &Vec<Vec<bool>>) -> Vec<usize> {
    let mut res: Vec<usize> = Vec::new();
    for i in 1..pattern.len() {
        if pattern[i] == pattern[i-1] {
            res.push(i);
        }
    }

    res
}

fn find_same_cols(pattern: &Vec<Vec<bool>>) -> Vec<usize> {
    let mut res: Vec<usize> = Vec::new();
    for i in 1..pattern[0].len() {
        let col_i: Vec<bool> = pattern.iter().map(|x| x[i]).collect();
        let col_j: Vec<bool> = pattern.iter().map(|x| x[i-1]).collect();

        if col_i == col_j {
            res.push(i)
        }
    }
    res
}

fn validate_row_pattern(pattern: &Vec<Vec<bool>>, mirror_idx: usize) -> bool {

    
    let mut i = mirror_idx - 1;
    let mut j = mirror_idx;

    while j < pattern.len() {

        if pattern[i] != pattern[j] {
            return false;
        }

        if i == 0 {
            break;
        }

        i -= 1;
        j += 1;
    }

    true
}

fn validate_col_pattern(pattern: &Vec<Vec<bool>>, mirror_idx: usize) -> bool {
    
    let mut i = mirror_idx - 1;
    let mut j = mirror_idx;

    while  j < pattern[0].len() {
        let col_i: Vec<bool> = pattern.iter().map(|x| x[i]).collect();
        let col_j: Vec<bool> = pattern.iter().map(|x| x[j]).collect();

        if col_i != col_j {
            return false;
        }

        if i == 0 {
            break;
        }

        i -= 1;
        j += 1;
    }

    true
}

fn process_pattern(pattern: &Vec<Vec<bool>>) -> usize {

    let mut mirror_indexes = find_same_rows(&pattern);

    if !mirror_indexes.is_empty() {
        
        for mirror_idx in mirror_indexes {
            if validate_row_pattern(&pattern, mirror_idx) {
                return 100 * mirror_idx
            }
        }
    }

    mirror_indexes = find_same_cols(&pattern);

    if !mirror_indexes.is_empty() {
        
        for mirror_idx in mirror_indexes {
            if validate_col_pattern(&pattern, mirror_idx) {
                return mirror_idx
            }
        }
    }

    panic!("Invalid pattern");

}